#version 330

struct Box
{
	vec3 mn; // Minimum coordinates
	vec3 mx; // Maximum coordinates
};

struct Ray
{
	vec3 orig; // Origin of ray
	vec3 dir; // Direction of ray
};

layout (std140) uniform Matrices
{
	mat4 m_pvm;
};

uniform float time;
uniform Box bbox = Box(vec3(-1.f), vec3(1.f));

in vec4 position;	// local space
in vec3 normal;		// local space

out vec4 color;

//----------------------------------------------------------------
// TODO:
// 1) Morphing as interpolation between two points (object and cube).
// 2) Implement ray box intersection test.
// 3) Visualize time (interpolation / extrapolation).
// 4) Prevent extrapolation - e.g. use sin function.
//----------------------------------------------------------------

// Intersect a ray with a box
bool intersectBox(Box box, Ray ray, out float tNear, out float tFar);

void main()
{
	color = vec4(vec3(0.f), 1.f);

	vec3 sph = position.xyz;
	// Base distance
	float tC = length(sph);

	// Distance to intersection

	// Move along the normalized vector from the center to the point.
	vec4 v = position;
	v.xyz = normalize(sph) * mix(tC, tFar, abs(sin(time)));
	gl_Position = m_pvm * v;

#if 0
	if(time > 1.f)
		color = vec4(1.f, 0.f, 0.f, 1.f);
	else
		color = vec4(0.f, 1.f, 0.f, 1.f);
#endif
}

// Intersect a ray with a box
bool intersectBox(Box box, Ray ray, out float near, out float far)
{
	// Ray plane distance in each axis: box - orig / dir
	vec3 hmin = (box.mn - ray.orig) / ray.dir;
	vec3 hmax = (box.mx - ray.orig) / ray.dir;

	// Minimum and maximum values in each axis (near / far in that plane)
	vec3 pmin = min(hmin, hmax);
	vec3 pmax = max(hmin, hmax);

	// Near / far in all axes
	near = max(max(pmin.x, max(pmin.y, pmin.z)), 0.f);
	far  = min(pmax.x, min(pmax.y, pmax.z));

	return far >= near;
}